'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var jsCore = require('@lorisleiva/js-core');
var beet = require('@metaplex-foundation/beet');

function _interopNamespace(e) {
  if (e && e.__esModule) return e;
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () { return e[k]; }
        });
      }
    });
  }
  n["default"] = e;
  return Object.freeze(n);
}

var beet__namespace = /*#__PURE__*/_interopNamespace(beet);

// Simple numbers.
const bool = () => wrapBeet(beet__namespace.bool);
const u8 = () => wrapBeet(beet__namespace.u8);
const u16 = () => wrapBeet(beet__namespace.u16);
const u32 = () => wrapBeet(beet__namespace.u32);
const i8 = () => wrapBeet(beet__namespace.i8);
const i16 = () => wrapBeet(beet__namespace.i16);
const i32 = () => wrapBeet(beet__namespace.i32);

// Big numbers.
const u64 = () => {
  const serializer = wrapBigintBeet(beet__namespace.u64);
  return {
    ...serializer,
    serialize: value => {
      if (value < 0) throw new RangeError('u64 cannot be negative');
      return serializer.serialize(value);
    }
  };
};
const u128 = () => {
  const serializer = wrapBigintBeet(beet__namespace.u128);
  return {
    ...serializer,
    serialize: value => {
      if (value < 0) throw new RangeError('u128 cannot be negative');
      return serializer.serialize(value);
    }
  };
};
const i64 = () => {
  const serializer = wrapBigintBeet(beet__namespace.i64);
  return {
    ...serializer,
    serialize: value => {
      if (value < (-2n) ** 63n) {
        throw new RangeError('i64 cannot be lower than -2^63');
      }
      if (value > 2n ** 63n - 1n) {
        throw new RangeError('i64 cannot be greater than 2^63 - 1');
      }
      return serializer.serialize(value);
    }
  };
};
const i128 = () => {
  const serializer = wrapBigintBeet(beet__namespace.i128);
  return {
    ...serializer,
    serialize: value => {
      if (value < (-2n) ** 127n) {
        throw new RangeError('i128 cannot be lower than -2^127');
      }
      if (value > 2n ** 127n - 1n) {
        throw new RangeError('i128 cannot be greater than 2^127 - 1');
      }
      return serializer.serialize(value);
    }
  };
};

// Helpers.
function wrapBeet(fixedBeet) {
  return {
    description: fixedBeet.description,
    serialize: value => {
      const buffer = Buffer.alloc(fixedBeet.byteSize);
      fixedBeet.write(buffer, 0, value);
      return new Uint8Array(buffer);
    },
    deserialize: (bytes, offset = 0) => {
      const buffer = Buffer.from(bytes);
      const value = fixedBeet.read(buffer, offset);
      return [value, offset + fixedBeet.byteSize];
    }
  };
}
function wrapBigintBeet(fixedBeet) {
  return {
    description: fixedBeet.description,
    serialize: value => {
      const buffer = Buffer.alloc(fixedBeet.byteSize);
      fixedBeet.write(buffer, 0, value);
      return new Uint8Array(buffer);
    },
    deserialize: (bytes, offset = 0) => {
      const buffer = Buffer.from(bytes);
      const rawValue = fixedBeet.read(buffer, offset);
      const value = typeof rawValue === 'number' ? BigInt(rawValue) : jsCore.toBigInt(rawValue.toString());
      return [value, offset + fixedBeet.byteSize];
    }
  };
}

exports.bool = bool;
exports.i128 = i128;
exports.i16 = i16;
exports.i32 = i32;
exports.i64 = i64;
exports.i8 = i8;
exports.u128 = u128;
exports.u16 = u16;
exports.u32 = u32;
exports.u64 = u64;
exports.u8 = u8;
//# sourceMappingURL=numbers.cjs.map
