import { DataEnumToSerializerTuple, DataEnumUnion, Option, PublicKey, PublicKeyInput, ScalarEnum, Serializer, SerializerInterface, StructToSerializerTuple, WrapInSerializer } from '@lorisleiva/js-core';
export declare class BeetSerializer implements SerializerInterface {
    tuple<T extends any[]>(items: [...WrapInSerializer<T>], description?: string): Serializer<T>;
    vec<T>(itemSerializer: Serializer<T>, description?: string): Serializer<T[]>;
    array<T>(itemSerializer: Serializer<T>, size: number, description?: string): Serializer<T[]>;
    map<K, V>(keySerializer: Serializer<K>, valueSerializer: Serializer<V>, description?: string): Serializer<Map<K, V>>;
    set<T>(itemSerializer: Serializer<T>, description?: string): Serializer<Set<T>>;
    option<T>(itemSerializer: Serializer<T>, description?: string): Serializer<Option<T>>;
    struct<T extends object>(fields: StructToSerializerTuple<T>, description?: string): Serializer<T>;
    enum<T>(constructor: ScalarEnum<T> & {}, description?: string): Serializer<T>;
    dataEnum<T extends DataEnumUnion>(fields: DataEnumToSerializerTuple<T>, description?: string): Serializer<T>;
    get unit(): Serializer<void>;
    get bool(): Serializer<boolean>;
    get u8(): Serializer<number>;
    get u16(): Serializer<number>;
    get u32(): Serializer<number>;
    get u64(): Serializer<number | bigint, bigint>;
    get u128(): Serializer<number | bigint, bigint>;
    get i8(): Serializer<number>;
    get i16(): Serializer<number>;
    get i32(): Serializer<number>;
    get i64(): Serializer<number | bigint, bigint>;
    get i128(): Serializer<number | bigint, bigint>;
    get f32(): Serializer<number>;
    get f64(): Serializer<number>;
    get string(): Serializer<string>;
    get bytes(): Serializer<Uint8Array>;
    get publicKey(): Serializer<PublicKey | PublicKeyInput, PublicKey>;
}
